package com.tzld.piaoquan.longarticle.service.impl;

import com.alibaba.fastjson.JSON;
import com.tzld.piaoquan.longarticle.dao.mapper.PlanAccountMapper;
import com.tzld.piaoquan.longarticle.dao.mapper.PublishMiniprogramMapper;
import com.tzld.piaoquan.longarticle.model.dto.*;
import com.tzld.piaoquan.longarticle.model.bo.*;
import com.tzld.piaoquan.longarticle.model.po.*;
import com.tzld.piaoquan.longarticle.model.po.PlanAccount;
import com.tzld.piaoquan.longarticle.model.vo.*;
import com.tzld.piaoquan.longarticle.service.CoreService;
import com.tzld.piaoquan.longarticle.utils.DateUtil;
import com.tzld.piaoquan.longarticle.utils.TimeZoneUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;

@Service
public class CoreServiceImpl implements CoreService {

    @Autowired
    private PlanServiceImpl planService;

    @Autowired
    private SortServiceImpl sortService;

    @Autowired
    private ContentServiceImpl contentService;

    @Autowired
    private PublishServiceImpl publishService;

    @Autowired
    private KimiServiceImpl kimiService;

    @Autowired
    private CardServiceImpl cardService;

    @Autowired
    private CoverServiceImpl coverService;

    @Autowired
    private PlanAccountMapper planAccountMapper;

    @Autowired
    private PlanAccountServiceImpl planAccountService;

    @Autowired
    private PublishMiniprogramMapper publishMiniprogramMapper;

    public void initPlanAccount() {
        List<LongArticleSystemPlan> allLongArticleSystemPlans = planService.getAllLongArticleSystemPlan();
        for (LongArticleSystemPlan longArticleSystemPlan : allLongArticleSystemPlans) {
            if (longArticleSystemPlan.getPublishRate() == 1) {
                List<Integer> weeks = JSON.parseArray(longArticleSystemPlan.getPublishDate(), Integer.class);
                if (!weeks.contains(TimeZoneUtil.getTodayDayOfWeek(TimeZoneUtil.Timezone.china))) {
                    continue;
                }
            }
            //获取排序策略
            Map<String, String> sortStgMap = new HashMap<>();
            List<GzhArticleSortTaskParam> gzhArticleSortTask = longArticleSystemPlan.getGzhArticleSortTask();
            for (GzhArticleSortTaskParam gzhArticleSortTaskParam : gzhArticleSortTask) {
                String strategy = gzhArticleSortTaskParam.getStrategy();
                List<String> accountIds = gzhArticleSortTaskParam.getAccountIds();
                for (String accountId : accountIds) {
                    sortStgMap.put(accountId, strategy);
                }
            }
            //获取小程序策略
            Map<String, MiniprogramInsertTaskParam> miniStgMap = new HashMap<>();
            List<MiniprogramInsertTaskParam> miniprogramInsertTasks = longArticleSystemPlan.getMiniprogramInsertTasks();
            for (MiniprogramInsertTaskParam miniprogramInsertTaskParam : miniprogramInsertTasks) {
                List<String> accountIds = miniprogramInsertTaskParam.getAccountIds();
                for (String accountId : accountIds) {
                    miniStgMap.put(accountId, miniprogramInsertTaskParam);
                }
            }
            for (LongArticleSystemPlanAccount account : longArticleSystemPlan.getAccountList()) {
                PlanAccountExample example = new PlanAccountExample();
                example.createCriteria().andAccountIdEqualTo(account.getId()).andPlanIdEqualTo(longArticleSystemPlan.getId())
                        .andCreateTimeGreaterThan(DateUtil.getThatDayDate());
                long l = planAccountMapper.countByExample(example);
                if (l > 0) {
                    continue;
                }
                PlanAccount planAccount = new PlanAccount();
                String accountId = account.getId();
                planAccount.setPlanId(longArticleSystemPlan.getId());
                planAccount.setPlanName(longArticleSystemPlan.getName());
                planAccount.setAccountId(accountId);
                planAccount.setAccountName(account.getName());
                planAccount.setGhId(account.getGzhId());
                planAccount.setPublishNum(longArticleSystemPlan.getPublishNum());
                planAccount.setPublishPreMinNum(longArticleSystemPlan.getPublishPerMinNum());
                MiniprogramInsertTaskParam miniprogramInsertTaskParam = miniStgMap.get(accountId);
                if (miniprogramInsertTaskParam != null) {
                    planAccount.setMiniprogramUseType(miniprogramInsertTaskParam.getMiniprogramUseType());
                    planAccount.setMiniStrategy(miniprogramInsertTaskParam.getStrategy());
                }
                if (sortStgMap.containsKey(accountId)) {
                    planAccount.setSortStrategy(sortStgMap.get(accountId));
                }
                planAccount.setPushType(longArticleSystemPlan.getPushType());
                planAccountMapper.insertSelective(planAccount);
            }
        }
    }

    public void matchContent() {
        //查询状态为0的请求匹配
        PlanAccountExample example = new PlanAccountExample();
        example.createCriteria().andCreateTimeGreaterThan(DateUtil.getThatDayDate()).andStatusEqualTo(0);
        List<PlanAccount> planAccounts = planAccountMapper.selectByExample(example);
        for (PlanAccount planAccount : planAccounts) {
            LongArticleSystemGetContentsParam param = new LongArticleSystemGetContentsParam();
            param.setAccountId(param.getAccountId());
            param.setPlanId(planAccount.getPlanId());
            LongArticleSystemContentVO longArticleSystemContentVO = planService.getContentItemList(param);
            if (longArticleSystemContentVO == null) {
                continue;
            }
            //没有待匹配的文章  更新状态为1  待排序
            if (longArticleSystemContentVO.getTotalCount() == 0) {
                planAccountService.updateStatus(1, planAccount.getId());
            }
            for (ContentItemVO contentItemVO : longArticleSystemContentVO.getContentItemList()) {
                MatchVideo content = contentService.getContent(planAccount.getGhId(), contentItemVO.getContent());
                if (content != null) {
                    Integer contentStatus = content.getContentStatus();
                    if (contentStatus == 3 || contentStatus == 4) {
                        MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                        statusParam.setStatus(2);
                        statusParam.setPublishContentId(contentItemVO.getPublishContentId());
                        planService.updateMatchMiniprogramStatus(statusParam);
                        continue;
                    }
                    if (contentStatus == 99 || contentStatus == 97) {
                        MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                        statusParam.setStatus(3);
                        statusParam.setPublishContentId(contentItemVO.getPublishContentId());
                        if (contentStatus == 99) {
                            statusParam.setErrorMsg("匹配视频失败");
                        }
                        if (contentStatus == 97) {
                            statusParam.setErrorMsg("文章晋级或者退场");
                        }
                        planService.updateMatchMiniprogramStatus(statusParam);
                        continue;
                    }
                }

                MiniprogramCardRequest request = new MiniprogramCardRequest();
                request.setGhId(planAccount.getGhId());
                request.setAccountName(planAccount.getAccountName());
                request.setContent(contentItemVO.getContent());
                request.setTitle(contentItemVO.getTitle());
                request.setStrategy(planAccount.getMiniStrategy());
                request.setArticleId(contentItemVO.getSourceId());
                request.setFlowPoolLevelTag(contentItemVO.getFlowPoolLevelTag());
                contentService.matchMiniprogramVideo(request);
            }
        }
    }

    public void core() {
//        List<PlanAccount> planAccounts = new ArrayList<>();
//
//        PlanAccount planAccount1 = new PlanAccount();
//        planAccount1.setAccountId("20231208111728193795195");
//        planAccount1.setAccountName("票圈大事件");
//        planAccount1.setGhId("gh_9e559b3b94ca");
//        planAccount1.setPlanId("20241012140830436890740");
//        planAccount1.setPublishNum(8);
//        planAccount1.setMiniStrategy("strategy_v1");
//        planAccount1.setSortStrategy("ArticleRankV5");
//        planAccount1.setPublishPreMinNum(4);
//        planAccount1.setMiniprogramUseType(1);
//        planAccount1.setPushType(2);
//        planAccounts.add(planAccount1);

        //查询状态为1的  开始排序  发布小程序视频  组装 发布文章
        PlanAccountExample example = new PlanAccountExample();
        //匹配完成和发布失败
        List<Integer> status = Arrays.asList(1, 5);
        example.createCriteria().andCreateTimeGreaterThan(DateUtil.getThatDayDate()).andStatusIn(status);
        List<PlanAccount> planAccounts = planAccountMapper.selectByExample(example);
        for (PlanAccount planAccount : planAccounts) {
            ArticleSortRequest articleSortRequest = new ArticleSortRequest();
            articleSortRequest.setAccountId(planAccount.getAccountId());
            articleSortRequest.setAccountName(planAccount.getAccountName());
            articleSortRequest.setGhId(planAccount.getGhId());
            articleSortRequest.setStrategy(planAccount.getSortStrategy());
            articleSortRequest.setPublishNum(planAccount.getPublishNum());
            articleSortRequest.setPlanId(planAccount.getPlanId());
            articleSortRequest.setPushType(planAccount.getPushType());

            ArticleSortResponse articleSortResponse = sortService.publishArticleSort(articleSortRequest);
            ArticleSortResponseData data = articleSortResponse.getData();
            if (data == null || CollectionUtils.isEmpty(data.getRank_list())) {
                continue;
            }
            if (!CollectionUtils.isEmpty(data.getFilter_list())) {
                List<FilterSortContentParam> list = new ArrayList<>();
                for (PublishArticleData publishArticleData : data.getFilter_list()) {
                    FilterSortContentParam param = new FilterSortContentParam();
                    param.setFilterReason(publishArticleData.getFilterReason());
                    param.setPublishContentId(publishArticleData.getId());
                    list.add(param);
                }
                FilterSortParam param = new FilterSortParam();
                param.setFilterSortContentList(list);
                planService.filterSortContents(param);
            }
            List<LongArticleSystemPushContentParam> pushContentList = new ArrayList<>();
            //排序结束
            List<PublishArticleData> rankList = data.getRank_list();
            for (PublishArticleData publishArticleData : rankList) {
                LongArticleSystemPushContentParam contentParam = new LongArticleSystemPushContentParam();
                String publishContentId = publishArticleData.getId();
                //获取小程序
                List<CrawlerVideo> contentVideos =
                        contentService.getContentVideo(publishArticleData.getSourceId(), publishArticleData.getContentPoolType(), planAccount.getGhId());
                List<VideoDetail> videoDetails = new ArrayList<>();
                //发布小程序
                for (CrawlerVideo crawlerVideo : contentVideos) {
                    String videoOssPath = crawlerVideo.getVideoOssPath();
                    String platform = crawlerVideo.getPlatform();
                    String userId = crawlerVideo.getUserId();
                    String traceId = crawlerVideo.getTraceId();
                    String kimiTitle;
                    LongArticlesText kimiText = kimiService.getKimiText(crawlerVideo.getContentId());
                    if (kimiText != null && StringUtils.isNotEmpty(kimiText.getKimiTitle())) {
                        kimiTitle = kimiText.getKimiTitle();
                    } else {
                        kimiTitle = crawlerVideo.getVideoTitle();
                    }
                    VideoDetail publish = publishService.publish(videoOssPath, userId, kimiTitle, platform, traceId);
                    if (publish == null) {
                        continue;
                    }
                    videoDetails.add(publish);
                }
                if (videoDetails.size() < 2) {
                    MatchMiniprogramStatusParam statusParam = new MatchMiniprogramStatusParam();
                    statusParam.setStatus(3);
                    statusParam.setPublishContentId(publishContentId);
                    statusParam.setErrorMsg("小程序视频数量不足");
                    planService.updateMatchMiniprogramStatus(statusParam);
                    continue;
                }
                String ghId = planAccount.getGhId();
                Long planAccountId = planAccount.getId();
                List<PublishMiniprogram> cardList = cardService.generateCards(planAccountId, ghId, videoDetails, planAccount.getMiniprogramUseType(), publishContentId);
                if (CollectionUtils.isEmpty(cardList) && cardList.size() < 2) {
                    continue;
                }
                List<PublishMiniprogramParam> publishCardList = new ArrayList<>();
                for (PublishMiniprogram publishMiniprogram : cardList) {
                    String videoCover = publishMiniprogram.getVideoCover();
                    String wxUrl = coverService.pushCover(videoCover, publishMiniprogram.getPublishContentId());
                    if (StringUtils.isEmpty(wxUrl)) {
                        continue;
                    }
                    PublishMiniprogramParam param = new PublishMiniprogramParam();
                    param.setProgramId(publishMiniprogram.getAppId());
                    param.setProgramName(publishMiniprogram.getAppName());
                    param.setProgramAvatar(publishMiniprogram.getAvatar());
                    param.setMini_id(Integer.valueOf(publishMiniprogram.getMiniId()));
                    param.setMiniprogramUseType(publishMiniprogram.getMiniProgramType());
                    String videoTitle = publishMiniprogram.getVideoTitle();
                    if (org.springframework.util.StringUtils.hasText(videoTitle)) {
                        videoTitle = videoTitle.replace("\"", "");
                        if (videoTitle.length() > 32) {
                            videoTitle = videoTitle.substring(0, 32);
                        }
                        param.setProductionName(videoTitle);
                    }
                    param.setProductionCover(wxUrl);
                    param.setProductionPath(publishMiniprogram.getProductionPath());
                    param.setRootShareId(publishMiniprogram.getRootShareId());
                    param.setSource(publishMiniprogram.getSource());
                    param.setVideoUrl(publishMiniprogram.getVideoPath());
                    param.setParagraphPosition(BigDecimal.valueOf(publishMiniprogram.getIndex() * 0.25));
                    if (publishMiniprogram.getIndex() == 1) {
                        param.setParagraphPosition(BigDecimal.valueOf(0.01));
                    }
                    param.setTraceId(publishMiniprogram.getTraceId());
                    publishCardList.add(param);
                    publishMiniprogramMapper.insertSelective(publishMiniprogram);
                }

                if (publishCardList.size() < 2) {
                    continue;
                }
                contentParam.setPublishContentId(publishContentId);
                contentParam.setMiniprogramCardList(publishCardList);
                pushContentList.add(contentParam);
            }
            LongArticleSystemCreatePushTaskParam gzhPushParam = new LongArticleSystemCreatePushTaskParam();
            gzhPushParam.setPlanId(planAccount.getPlanId());
            gzhPushParam.setAccountId(planAccount.getAccountId());
            gzhPushParam.setPushType(planAccount.getPushType());
            gzhPushParam.setPushScheduleTimestamp(System.currentTimeMillis());
            if (StringUtils.isNotEmpty(planAccount.getPublishPushTime())) {
                String dateTimeStr = TimeZoneUtil.todayYMD(TimeZoneUtil.Timezone.china) + " " + planAccount.getPublishPushTime();
                gzhPushParam.setPushScheduleTimestamp(TimeZoneUtil.getDateStrTimestamp(dateTimeStr, "yyyy-MM-dd HH:mm:ss", TimeZoneUtil.Timezone.china));
            } else {
                gzhPushParam.setPushScheduleTimestamp(System.currentTimeMillis());
            }
            gzhPushParam.setPushContentList(pushContentList);
            if (pushContentList.size() < planAccount.getPublishPreMinNum()) {
                //TODO 数量不足 更新状态 发布失败
                planAccountService.updateStatus(5, planAccount.getId());
                continue;
            }
            System.out.println(gzhPushParam);
            String pushId = planService.createPushTask(gzhPushParam);
            if (StringUtils.isNotEmpty(pushId)) {
                //更新状态  发布完成
                PlanAccount updatePlanAccount = new PlanAccount();
                updatePlanAccount.setId(planAccount.getId());
                updatePlanAccount.setStatus(3);
                updatePlanAccount.setPushId(pushId);
                planAccountMapper.updateByPrimaryKey(updatePlanAccount);
                planAccountService.updateStatus(3, planAccount.getId());
            }
        }
    }

    public void getPushStatus() {
        PlanAccountExample example = new PlanAccountExample();
        //匹配完成和发布失败
        List<Integer> status = Arrays.asList(3);
        example.createCriteria().andCreateTimeGreaterThan(DateUtil.getThatDayDate()).andStatusIn(status);
        List<PlanAccount> planAccounts = planAccountMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(planAccounts)) {
            return;
        }
        for (PlanAccount planAccount : planAccounts) {
            if (StringUtils.isNotEmpty(planAccount.getPublishPushTime())) {
                String dateTimeStr = TimeZoneUtil.todayYMD(TimeZoneUtil.Timezone.china) + " " + planAccount.getPublishPushTime();
                long dateStrTimestamp = TimeZoneUtil.getDateStrTimestamp(dateTimeStr, "yyyy-MM-dd HH:mm:ss", TimeZoneUtil.Timezone.china);
                if (dateStrTimestamp < new Date().getTime()) {
                    continue;
                }
            }
            if (StringUtils.isEmpty(planAccount.getPushId())) {
                continue;
            }
            PushStatusParam param = new PushStatusParam();
            param.setPushId(planAccount.getPushId());
            PushStatusVO pushStatusVO = planService.queryPushStatus(param);
            int updateStatus = 0;
            //推送失败  重新推送
            if (pushStatusVO.getPushStatus() == 3) {
                updateStatus = 5;
            }
            //推送成功
            if (pushStatusVO.getPushStatus() == 2) {
                //发表失败
                if (pushStatusVO.getPublishStatus() == 3) {
                    updateStatus = 5;
                }
                //发表成功
                if (pushStatusVO.getPublishStatus() == 2) {
                    updateStatus = 4;
                }
            }
            //更新状态  4成功  5失败
            if (updateStatus != 0) {
                planAccountService.updateStatus(updateStatus, planAccount.getId());
            }
        }
    }
}
